﻿//
//   (c) Copyright Agilent Technologies 2012
//              All rights reserved.
//
//  Customer shall have the personal, non-
// transferable rights to use, copy or modify
// this SAMPLE PROGRAM for Customer's internal
// operations. Customer shall use the SAMPLE
// PROGRAM solely and exclusively for its own
// purpose and shall not license, lease, market
// or distribute the SAMPLE PROGRAM or modification
// or any part thereof.
//
//  Agilent shall not be liable for the quality,
// performance or behavior of the SAMPLE PROGRAM.
// Agilent especially disclaims that the operation
// of the SAMPLE PROGRAM shall be uninterrupted or
// error free. This SAMPLE PROGRAM is provided
// AS IS.
//
//  AGILENT DISCLAIMS THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
//  Agilent shall not be liable for any infringement
// of any patent, trademark, copyright or other
// proprietary rights by the SAMPLE PROGRAM or
// its use. Agilent does not warrant that the SAMPLE
// PROGRAM is free from infringements or such
// rights of third parties. However, Agilent will not
// knowingly infringe or deliver a software that
// infringes the patent, trademark, copyright or
// other proprietary right of a third party.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ivi.Visa.Interop;

namespace B2900CsSample
{
    class DnvmControl
    {
        private ResourceManager ioMgr = new ResourceManager();
        private FormattedIO488 instrument = null;
        private string visaResStr = "";
        private string modelName = "";
        private string errorMsg = "";
        private const int WAIT_BEFORE_QUERY = 10;

        private int countData = 0;          // Num of Measure Count
        private double delayTime = 0.0001;  // Measure delay : 100us  [0-3600sec]
        private double nplc = 0.2;          // Integragion time : 0.2


        public string ModelName
        {
            get { return modelName; }
        }

        public string ErrorMessage
        {
            get { return errorMsg; }
        }



        /// <summary>
        /// Open connection with 34420A
        /// </summary>
        /// <param name="address"></param>
        /// <returns></returns>
        public bool OpenDevice(string address)
        {
            try
            {
                if (instrument != null)
                {
                    CloseDevice();
                }

                string buf;

                // Open
                visaResStr = address;
                instrument = new FormattedIO488();
                instrument.IO = (IMessage)ioMgr.Open(address);
                instrument.IO.TerminationCharacterEnabled = true;
                System.Threading.Thread.Sleep(200);

                // Check Model Name
                instrument.WriteString("*IDN?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                buf = instrument.ReadString().Trim();
                string[] tokens = buf.Split(',');
                if (tokens.Length < 2)
                {
                    modelName = "";
                    errorMsg = "Illegal *IDN response.";
                    CloseDevice();
                    return false;
                }
                modelName = buf.Split(',')[1];

                // Reset
                instrument.WriteString("*RST");
                instrument.WriteString("*CLS");
            }
            catch (Exception ex)
            {
                instrument = null;
                errorMsg = ex.Message;
                return false;
            }

            return true;
        }


        /// <summary>
        /// Close connection with 34420A
        /// </summary>
        public void CloseDevice()
        {
            if (instrument != null) instrument.IO.Close();
            instrument = null;
        }


        /// <summary>
        /// Initiates 34420A
        /// </summary>
        /// <param name="countdata"></param>
        /// <param name="Npl"></param>
        /// <returns></returns>
        public bool Initiate(int countdata, double Npl)
        {
            countData = countdata;
            nplc = Npl;

            try
            {
                instrument.WriteString("TRIG:SOUR EXT");                        // Select external trigger source
                instrument.WriteString("TRIG:COUN " + countData.ToString());    // The number of triggers
                instrument.WriteString("TRIG:DEL " + delayTime.ToString());     // a trigger delay between the trigger signal and each sample that follows.
                instrument.WriteString("VOLT:DC:NPLC " + nplc.ToString());      // Integragion Time.

                instrument.WriteString("INIT");                                 // Wakeup and wait Trigger.

                return true;
            }
            catch (Exception ex)
            {
                instrument = null;
                errorMsg = ex.Message;
                return false;
            }
        }


        /// <summary>
        /// Get measure results.
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public bool GetData(double[] data)
        {
            try
            {
                string ret;
                int points;

                instrument.WriteString("DATA:POIN?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                points = int.Parse(instrument.ReadString().Trim());

                instrument.WriteString("FETC?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                ret = instrument.ReadString().Trim();
                string[] tokens = ret.Split(',');

                for (int i = 0; i < tokens.Length; i++)
                {
                    data[i] = double.Parse(tokens[i]);
                }

                return true;
            }
            catch (Exception ex)
            {
                errorMsg = ex.Message;
                return false;
            }
        }
    }
}
